/* BdfServer - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.options;

import fr.exemole.bdfserver.api.ui.options.InputUiOption;


/**
 *
 * @author Vincent Calame
 */
public class InputUiOptionBuilder {

    private String inputType = InputUiOption.INPUT_TEXT;
    private String widthType = InputUiOption.WIDTH_LARGE;
    private int rows = 1;

    public InputUiOptionBuilder() {

    }

    public InputUiOptionBuilder(InputUiOption inputUiOption) {
        this.inputType = inputUiOption.getInputType();
        this.widthType = inputUiOption.getWidthType();
        this.rows = inputUiOption.getRows();
    }

    public InputUiOptionBuilder setInputType(String inputType) {
        this.inputType = InputUiOption.checkInputType(inputType);
        return this;
    }

    public InputUiOptionBuilder setWidthType(String widthType) {
        this.widthType = InputUiOption.checkWidthType(widthType);
        return this;
    }

    public InputUiOptionBuilder setRows(int rows) {
        if (rows <= 0) {
            throw new IllegalArgumentException("rows <= 0");
        }
        this.rows = rows;
        return this;
    }

    public InputUiOption toInputUiOption() {
        return new InternalInputUiOption(inputType, widthType, rows);
    }

    public static InputUiOptionBuilder init(InputUiOption inputUiOption) {
        return new InputUiOptionBuilder(inputUiOption);
    }

    public static InputUiOptionBuilder init() {
        return new InputUiOptionBuilder();
    }

    public static InputUiOption build(String inputType, String widthType, int rows) {
        return InputUiOptionBuilder.init()
                .setInputType(inputType)
                .setWidthType(widthType)
                .setRows(rows)
                .toInputUiOption();
    }


    private static class InternalInputUiOption implements InputUiOption {

        private final String inputType;
        private final String widthType;
        private final int rows;

        public InternalInputUiOption(String inputType, String widthType, int rows) {
            this.inputType = inputType;
            this.widthType = widthType;
            this.rows = rows;
        }

        @Override
        public String getInputType() {
            return inputType;
        }

        @Override
        public String getWidthType() {
            return widthType;
        }

        @Override
        public int getRows() {
            return rows;
        }

    }

}
